//
//  GSAPI.h
//  GigyaGSAPITester
//
//  Created by Moshe Shitrit on 7/27/10.
//  Copyright 2010 Mitug Distributed Systems Ltd. All rights reserved.
//

#import <Foundation/Foundation.h>
#import	"GSResponse.h"
#import	"GSSession.h"

/*
 This delegate can be optionally passed to GSAPI (by setting gsapi.eventDelegate)
 if the developer wants to know when login/logout/connect/disconnect occurred.
 NOTE: all protocol messages are optional
 */
@protocol GSEventDelegate <NSObject>
//@optional
-(void)	gsDidLogin:(NSString*)provider user:(GSDictionary*)user context:(id)context;
-(void)	gsDidLogout;
-(void)	gsDidAddConnection:(NSString*)provider user:(GSDictionary*)user context:(id)context;
-(void)	gsDidRemoveConnection:(NSString*)provider context:(id)context;
@end

/*
 delegate for showConnectUI
 NOTE: all protocol messages are optional
 */
@protocol GSAddConnectionsUIDelegate	<NSObject>
@optional
// fired when network selection page is shown 
-(void)	gsAddConnectionsUIDidLoad:(id)context; 

// fired when the login UI is closed (for any reason - canceled, error , operation completed OK)
-(void)	gsAddConnectionsUIDidClose:(id)context; 

// fired when error occurred, either from webView or as result of login process, or get user info that followed it
-(void)	gsAddConnectionsUIDidFail:(int)errorCode errorMessage:(NSString*)errorMessage context:(id)context; 

// fired when add connection operation (and getuserInfo that folllows it) complete
-(void)	gsAddConnectionsUIDidConnect:(NSString*)provider user:(GSDictionary*)user context:(id)context;


@end

/*
 delegate for showLoginUI
 NOTE: all protocol messages are optional
 */
@protocol GSLoginUIDelegate <NSObject>
@optional
// fired when network selection page is shown 
-(void)	gsLoginUIDidLoad:(id)context;

// fired when the login UI is closed (for any reason - canceled, error , operation completed OK)
-(void)	gsLoginUIDidClose:(id)context;

// fired when error occurred, either from webView or as result of login process, or get user info that followed it
-(void)	gsLoginUIDidFail:(int)errorCode errorMessage:(NSString*)errorMessage context:(id)context;

// fired when login operation (and getuserInfo that folllows it) complete
-(void)	gsLoginUIDidLogin:(NSString*)provider user:(GSDictionary*)user context:(id)context;
@end

/*
 delegate for any non-ui operation:
 login, addConnection, removeConnection, sendRequest
 NOTE: all protocol messages are optional
 */
@protocol GSResponseDelegate <NSObject>
- (void) gsDidReceiveResponse:(NSString*)method response:(GSResponse*)response context:(id)context;
@end


@interface GSAPI : NSObject  {
	@private
	id	apiInternal;
	@public
	id<GSEventDelegate>	eventDelegate;
}
@property (nonatomic, retain) 	id	apiInternal;
@property (nonatomic, assign) 	id<GSEventDelegate>	eventDelegate;

// initialize the library with your apiKey and the main view
-(id) initWithAPIKey:(NSString*)apiKey viewController:(UIViewController*)mainViewController; 

// login to a spcific provider
-(void) login:(GSDictionary*)params delegate:(id<GSResponseDelegate>)delegate context:(id)context;

// logout
-(void) logout;

// add a connection to a provider
-(void) addConnection:(GSDictionary*)params delegate:(id<GSResponseDelegate>)delegate context:(id)context;

// remove an existing connection
-(void) removeConnection:(GSDictionary*)params delegate:(id<GSResponseDelegate>)delegate context:(id)context;

// show provider slection UI for login
-(void) showLoginUI:(GSDictionary*)params delegate:(id<GSLoginUIDelegate>)delegate context:(id)context;

// show provider slection UI for adding a connection
-(void) showAddConnectionsUI:(GSDictionary *)params delegate:(id<GSAddConnectionsUIDelegate>)delegate context:(id)context;

// send a request (e.g. "getUserInfo", "getfriends", "publishUserAction" etc.)
-(void) sendRequest:(NSString*)method params:(GSDictionary*)params	useHTTPS:(BOOL)useHTTPS delegate:(id<GSResponseDelegate>)delegate context:(id)context;
-(void) sendRequest:(NSString*)method params:(GSDictionary*)params delegate:(id<GSResponseDelegate>)delegate context:(id)context;

// get the current session. returns nil if session doesn't exist
-(GSSession *) getSession;
@end








