//
//  GSAPI.h
//  GigyaGSAPITester
//
//  Created by Moshe Shitrit on 7/16/10.
//  Copyright 2010 Mitug Distributed Systems Ltd. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "SBJSON.h"
#import	"GSSession.h"
#import	"GSResponse.h"
#import	"GSDictionary.h"

#define GSAPI_NET_SELECT_URL @"gsapi://network_selected/"
#define GSAPI_UI_URL @"http://socialize.gigya.com/gs/mobile/loginui.aspx"
#define GSAPI_RESULT_URL @"gsapi://login_result/"
#define GSAPI_LOGIN_URL @"http://socialize.gigya.com/socialize.login"
#define GSAPI_CONNECT_URL @"http://socialize.gigya.com/socialize.addConnection"
#define GSAPI_TITLE_URL @"gsapi://set_title/"

#define GSAPI_PARAM_NAME_PROVIDER	@"provider"
#define GSAPI_PARAM_NAME_PROVIDER_DISPLAY_NAME	@"displayName"
#define GSAPI_PARAM_NAME_CID	@"cid"
#define GSAPI_PARAM_NAME_LANG	@"lang"
#define GSAPI_PARAM_NAME_ERROR_DESCRIPTION	@"error_description"
#define GSAPI_PARAM_NAME_REQUEST_PARAMS_DICTIONARY @"RequestParamsDict"
#define GSAPI_PARAM_NAME_GSRESPONSE @"GSResponse"
#define GSAPI_PARAM_NAME_SEND_REQUEST_METHOD @"SendRequestMethod"
#define GSAPI_PARAM_NAME_CONEXT @"Context"
#define GSAPI_PARAM_NAME_CAPTION_TEXT @"captionText"
#define GSAPI_PARAM_NAME_EXTRA_PERMISSIONS @"extraPermissions"


#define GetUserInfoCompletedAfterLogin_Notification @"GetUserInfoCompletedAfterLogin_Notification"
#define GetUserInfoCompletedAfterConnect_Notification @"GetUserInfoCompletedAfterConnect_Notification"
#define LoginCompleted_Notification @"LoginCompleted_Notification"
#define LogoutCompleted_Notification @"LogoutCompleted_Notification"
#define AddConnectionCompleted_Notification @"AddConnectionCompleted_Notification"
#define RemoveConnectionCompleted_Notification @"RemoveConnectionCompleted_Notification"
#define ShowLoginUICompleted_Notification @"ShowLoginUICompleted_Notification"
#define SendRequestCompleted_Notification @"SendRequestCompleted_Notification"
#define ShowAddConnectionsUICompleted_Notification @"ShowAddConnectionsUICompleted_Notification"
#define ShowAddConnectionsUILoaded_Notification @"ShowAddConnectionsUILoaded_Notification"
#define ShowLoginUILoaded_Notification @"ShowLoginUILoaded_Notification"
#define ShowAddConnectionsUIFailed_Notification @"ShowAddConnectionsUIFailed_Notification"
#define ShowLoginUIFailed_Notification @"ShowLoginUIFailed_Notification"
#define ShowAddConnectionsUIClosed_Notification @"ShowAddConnectionsUIClosed_Notification"
#define ShowLoginUIClosed_Notification @"ShowLoginUIClosed_Notification"



// @protocol GSWebViewControllerDelegate;

@protocol GSWebViewControllerDelegate
- (void)GSWebViewControllerDidFinish:(UIViewController *)controller;
@end

@protocol GSAPIDelegate
- (void)	onLoginCompleted:(GSDictionary *)pRequestParams response:(GSResponse *)pResponse context:(id)pContext;
- (void)	onLogoutCompleted:(GSResponse *)pResponse;
- (void)	onAddConnectionCompleted:(GSDictionary *)pParams response:(GSResponse *)pResponse context:(id)pContext;
- (void)	onRemoveConnectionCompleted:(GSDictionary *)pParams response:(GSResponse *)pResponse context:(id)pContext;
-	(void)	onShowLoginUICompleted:(GSDictionary *)pParams response:(GSResponse *)pResponse context:(id)pContext;
-	(void)	onShowLoginUILoaded:(GSDictionary *)pParams response:(GSResponse *)pResponse context:(id)pContext;
-	(void)	onShowLoginUIFailed:(GSDictionary *)pParams response:(GSResponse *)pResponse context:(id)pContext;
-	(void)	onShowLoginUIClosed:(GSDictionary *)pParams response:(GSResponse *)pResponse context:(id)pContext;
-	(void)	onShowAddConnectionsUICompleted:(GSDictionary *)pParams response:(GSResponse *)pResponse context:(id)pContext;
-	(void)	onShowAddConnectionsUILoaded:(GSDictionary *)pParams response:(GSResponse *)pResponse context:(id)pContext;
-	(void)	onShowAddConnectionsUIFailed:(GSDictionary *)pParams response:(GSResponse *)pResponse context:(id)pContext;
-	(void)	onShowAddConnectionsUIClosed:(GSDictionary *)pParams response:(GSResponse *)pResponse context:(id)pContext;
-	(void)	onSendRequestCompleted:(NSString	*)pMethod Params:(GSDictionary		*)pParams	response:(GSResponse *)pResponse  context:(id)pContext;
@end



typedef enum 
{
	GSEventType_LOGIN,
	GSEventType_LOGOUT,
	GSEventType_CONNECT,
	GSEventType_DISCONNECT
} GSEventType;

typedef enum
{
	GSWebAction_NavigateToLogin,
	GSWebAction_NavigateToProviderSelector,
	GSWebAction_NavigateToConnect
} GSWebActionType;


enum GSInternalErrorCode
{
	GSErrorCode_CancelledByUser = 400007,
	GSErrorCode_MissingAPIKey = 400002,
	GSErrorCode_NonReEntrantCallPending = 400002,
	GSErrorCode_InvalidAPIMethod = 400002
};

@interface GSAPI_Internal : NSObject <GSWebViewControllerDelegate>{
	
@private
	
	id <GSAPIDelegate> delegate;
	
	NSMutableDictionary	*m_pSettings;
	NSString	*m_pAPIKey;
	UIViewController	*m_pMainViewController;
	GSSession	*m_pSession;
	NSString	*m_pNotificationMessage;
	BOOL	m_bNoReEntrantCallPending;
	GSDictionary	*m_pSavedRequestParams;
}
@property (nonatomic, assign) id <GSAPIDelegate> delegate;
@property (nonatomic, retain) UIViewController *m_pMainViewController;
@property (nonatomic, retain) NSString *m_pAPIKey;
@property (nonatomic, retain) NSMutableDictionary	*m_pSettings;
@property (nonatomic, retain) GSSession	*m_pSession;
@property (nonatomic, retain) NSString	*m_pNotificationMessage;
@property (nonatomic) BOOL	m_bNoReEntrantCallPending;
@property (nonatomic, retain) GSDictionary	*m_pSavedRequestParams;


- (id)initWithAPIKey:(NSString *)APIKey ViewController:(UIViewController *)MainViewController delegate:(id<GSAPIDelegate>)pDelegate;
- (void) OnNotification:(NSNotification *)pNotification;
- (void) Login:(GSDictionary *)pParams	context:(id)pContext;
-	(void) Logout;
- (void) AddConnection:(GSDictionary *)pParams	context:(id)pContext;
- (void) RemoveConnection:(GSDictionary *)pParams	context:(id)pContext;
-	(void) ShowLoginUI:(GSDictionary *)pParams	context:(id)pContext;
-	(void) ShowAddConnectionsUI:(GSDictionary *)pParams	context:(id)pContext;
-	(void) SendRequest:(NSString	*)pMethod Params:(GSDictionary		*)pParams	UseHTTPS:(BOOL)bUseHTTPS	context:(id)pContext;

-	(void) GetUserInfo:(NSString *)pNotificationName	context:(id)pContext originalRequest:(GSDictionary *)pRequestParams;

-(void) LoadSession;
-(void) SaveSession;
-(void) LoadSettings;
-(void) SaveSettings;
-(void) SetSession:(GSSession *)pSession;
-(NSString	*) GetLastLoginProvider;
-(void) SetLastLoginProvider:(NSString	*)pProvider;

-	(void)	SendNotification:(NSString *)pMessage APIMethod:(NSString *)pMethod request:(GSDictionary		*)pRequestParams response:(GSResponse *)pResponse context:(id)pContext;
-	(void)	SendNotification:(NSString *)pMethod request:(GSDictionary		*)pRequestParams response:(GSResponse *)pResponse context:(id)pContext;

-	(NSString	*) GetAccessToken;
-	(NSString	*) GetSecretKey;
-(GSSession *) GetSession;

@end
